@@ -74,7 +74,7 @@ extern QPainter foreGroundPaint;
 
 /** Routine to print characters.  */
 
-void dviWindow::set_char(unsigned int cmd, unsigned int ch)
+void dviRenderer::set_char(unsigned int cmd, unsigned int ch)
 {
 #ifdef DEBUG_RENDER
   kdDebug(4300) << "set_char #" << ch << endl;
@@ -99,12 +99,12 @@ void dviWindow::set_char(unsigned int cmd, unsigned int ch)
   
   // Are we drawing text for a hyperlink? And are hyperlinks
   // enabled?
-  if (HTML_href != NULL && _showHyperLinks != 0) {
+  if (HTML_href != NULL) {
     // Now set up a rectangle which is checked against every mouse
     // event.
     if (line_boundary_encountered == true) {
       // Set up hyperlink
-      DVI_Hyperlink dhl;
+      Hyperlink dhl;
       dhl.baseline = currinf.data.pxl_v;
       dhl.box.setRect(x, y, pix.width(), pix.height());
       dhl.linkText = *HTML_href;
@@ -122,7 +122,7 @@ void dviWindow::set_char(unsigned int cmd, unsigned int ch)
     // event.
     if (line_boundary_encountered == true) {
       // Set up source hyperlinks
-      DVI_Hyperlink dhl;
+      Hyperlink dhl;
       dhl.baseline = currinf.data.pxl_v;
       dhl.box.setRect(x, y, pix.width(), pix.height());
       if (source_href != NULL) 
@@ -140,7 +140,7 @@ void dviWindow::set_char(unsigned int cmd, unsigned int ch)
   // search, etc.). Set up the currentlyDrawnPage->textLinkList.
   if (line_boundary_encountered == true) {
     // Set up source hyperlinks
-    DVI_Hyperlink link;
+    Hyperlink link;
     link.baseline = currinf.data.pxl_v;
     link.box.setRect(x, y, pix.width(), pix.height());
     link.linkText = "";
@@ -198,21 +198,21 @@ void dviWindow::set_char(unsigned int cmd, unsigned int ch)
     currinf.data.dvi_h = dvi_h_sav;
   else
     currinf.data.dvi_h += (int)(currinf.fontp->scaled_size_in_DVI_units * dviFile->getCmPerDVIunit() * 
-				(MFResolutions[font_pool.getMetafontMode()] / 2.54)/16.0 * g->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
+				(1200.0 / 2.54)/16.0 * g->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
 
   word_boundary_encountered = false;
   line_boundary_encountered = false;
 }
 
-void dviWindow::set_empty_char(unsigned int, unsigned int)
+void dviRenderer::set_empty_char(unsigned int, unsigned int)
 {
   return;
 }
 
-void dviWindow::set_vf_char(unsigned int cmd, unsigned int ch)
+void dviRenderer::set_vf_char(unsigned int cmd, unsigned int ch)
 {
 #ifdef DEBUG_RENDER
-  kdDebug(4300) << "dviWindow::set_vf_char( cmd=" << cmd << ", ch=" << ch << " )" << endl;
+  kdDebug(4300) << "dviRenderer::set_vf_char( cmd=" << cmd << ", ch=" << ch << " )" << endl;
 #endif
 
   static unsigned char   c;
@@ -237,7 +237,7 @@ void dviWindow::set_vf_char(unsigned int cmd, unsigned int ch)
   Q_UINT8 *end_ptr_sav      = end_pointer;
   command_pointer           = m->pos;
   end_pointer               = m->end;
-  draw_part(currinf.fontp->scaled_size_in_DVI_units*(dviFile->getCmPerDVIunit() * MFResolutions[font_pool.getMetafontMode()] / 2.54)/16.0, true);
+  draw_part(currinf.fontp->scaled_size_in_DVI_units*(dviFile->getCmPerDVIunit() * 1200.0 / 2.54)/16.0, true);
   command_pointer           = command_ptr_sav;
   end_pointer               = end_ptr_sav;
   currinf = oldinfo;
@@ -246,14 +246,14 @@ void dviWindow::set_vf_char(unsigned int cmd, unsigned int ch)
     currinf.data.dvi_h = dvi_h_sav;
   else
     currinf.data.dvi_h += (int)(currinf.fontp->scaled_size_in_DVI_units * dviFile->getCmPerDVIunit() * 
-				(MFResolutions[font_pool.getMetafontMode()] / 2.54)/16.0 * m->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
+				(1200.0 / 2.54)/16.0 * m->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
 }
 
 
-void dviWindow::set_no_char(unsigned int cmd, unsigned int ch)
+void dviRenderer::set_no_char(unsigned int cmd, unsigned int ch)
 {
 #ifdef DEBUG_RENDER
-  kdDebug(4300) << "dviWindow::set_no_char( cmd=" << cmd << ", ch =" << ch << " )"  << endl;
+  kdDebug(4300) << "dviRenderer::set_no_char( cmd=" << cmd << ", ch =" << ch << " )"  << endl;
 #endif
   
   if (currinf._virtual) {
@@ -270,7 +270,7 @@ void dviWindow::set_no_char(unsigned int cmd, unsigned int ch)
 }
 
 
-void dviWindow::draw_part(double current_dimconv, bool is_vfmacro)
+void dviRenderer::draw_part(double current_dimconv, bool is_vfmacro)
 {
 #ifdef DEBUG_RENDER
   kdDebug(4300) << "draw_part" << endl;
@@ -280,7 +280,7 @@ void dviWindow::draw_part(double current_dimconv, bool is_vfmacro)
   Q_UINT8 ch;
 
   currinf.fontp        = NULL;
-  currinf.set_char_p   = &dviWindow::set_no_char;
+  currinf.set_char_p   = &dviRenderer::set_no_char;
 
   for (;;) {
     ch = readUINT8();
@@ -357,8 +357,8 @@ void dviWindow::draw_part(double current_dimconv, bool is_vfmacro)
 	    line_boundary_encountered = true;
 	  }
 	  command_pointer += 11 * 4;
-	  currinf.data.dvi_h = MFResolutions[font_pool.getMetafontMode()] << 16; // Reminder: DVI-coordinates start at (1",1") from top of page
-	  currinf.data.dvi_v = MFResolutions[font_pool.getMetafontMode()];
+	  currinf.data.dvi_h = 1200 << 16; // Reminder: DVI-coordinates start at (1",1") from top of page
+	  currinf.data.dvi_v = 1200;
 	  currinf.data.pxl_v = int(currinf.data.dvi_v/shrinkfactor);
 	  currinf.data.w = currinf.data.x = currinf.data.y = currinf.data.z = 0;
 	  break;
@@ -569,7 +569,7 @@ void dviWindow::draw_part(double current_dimconv, bool is_vfmacro)
 }
 
 
-void dviWindow::draw_page(void)
+void dviRenderer::draw_page(void)
 {
   // Reset a couple of variables
   HTML_href         = 0;
@@ -599,17 +599,12 @@ void dviWindow::draw_page(void)
   kdDebug(4300) <<"draw_page" << endl;
 #endif
 
-  foreGroundPaint.fillRect(currentlyDrawnPixmap.rect(), PS_interface->getBackgroundColor(current_page) );
+  foreGroundPaint.fillRect( foreGroundPaint.viewport(), PS_interface->getBackgroundColor(current_page) );
 
   // Render the PostScript background, if there is one.
-  if (_postscript) {
-    QPixmap *pxm = PS_interface->graphics(current_page);
-    if (pxm != NULL) {
-      foreGroundPaint.drawPixmap(0, 0, *pxm);
-      delete pxm;
-    }
-  }
-
+  if (_postscript)
+    PS_interface->graphics(current_page, resolutionInDPI, dviFile->getMagnification(), foreGroundPaint);
+  
   // Now really write the text
   if (dviFile->page_offset.isEmpty() == true)
     return;
@@ -618,13 +613,12 @@ void dviWindow::draw_page(void)
     end_pointer     = dviFile->dvi_Data() + dviFile->page_offset[current_page+1];
   } else
     command_pointer = end_pointer = 0;
-
+  
   memset((char *) &currinf.data, 0, sizeof(currinf.data));
   currinf.fonttable      = &(dviFile->tn_table);
   currinf._virtual       = 0;
-
-  double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 
-    MFResolutions[font_pool.getMetafontMode()]/2.54;
+  
+  double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 1200.0/2.54;
   
   draw_part(65536.0*fontPixelPerDVIunit, false);
   if (HTML_href != 0) {
@@ -635,16 +629,4 @@ void dviWindow::draw_page(void)
     delete source_href;
     source_href = 0;
   }
-
-  // Mark hyperlinks in blue. We draw a blue line under the
-  // character whose width is equivalent to 0.5 mm, but at least
-  // one pixel.
-  int h = (int)(MFResolutions[font_pool.getMetafontMode()]*0.05/(2.54*shrinkfactor) + 0.5);
-  h = (h < 1) ? 1 : h;
-  for(unsigned int i=0; i<currentlyDrawnPage->hyperLinkList.size(); i++) {
-    int x = currentlyDrawnPage->hyperLinkList[i].box.left();
-    int w = currentlyDrawnPage->hyperLinkList[i].box.width();
-    int y = currentlyDrawnPage->hyperLinkList[i].baseline;
-    foreGroundPaint.fillRect(x, y, w, h, Qt::blue);
-  }
 }
