@@ -2,7 +2,7 @@
 //
 // Part of KDVI - A DVI previewer for the KDE desktop environemt 
 //
-// (C) 2003 Stefan Kebekus
+// (C) 2003--2004 Stefan Kebekus
 // Distributed under the GPL
 
 #include "dviwin.h"
@@ -15,6 +15,7 @@
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kmimetype.h>
 #include <kprocess.h>
 #include <kprocio.h>
 #include <kprogress.h>
@@ -37,10 +38,10 @@ extern void parse_special_argument(QString strg, const char *argument_name, int
 
 
 
-void dviWindow::prescan_embedPS(char *cp, Q_UINT8 *beginningOfSpecialCommand)
+void dviRenderer::prescan_embedPS(char *cp, Q_UINT8 *beginningOfSpecialCommand)
 {
 #ifdef  DEBUG_PRESCAN
-  kdDebug(4300) << "dviWindow::prescan_embedPS( cp = " << cp << " ) " << endl;
+  kdDebug(4300) << "dviRenderer::prescan_embedPS( cp = " << cp << " ) " << endl;
 #endif
 
   // Encapsulated Postscript File
@@ -63,9 +64,15 @@ void dviWindow::prescan_embedPS(char *cp, Q_UINT8 *beginningOfSpecialCommand)
   // macro packages (but not by others). This probably means that
   // graphic files are no longer found if the filename really does
   // contain quotes, but we don't really care that much.
-  if ((EPSfilename.at(0) == '\"') && (EPSfilename.at(EPSfilename.length()-1) == '\"')) {
+  if ((EPSfilename.at(0) == '\"') && (EPSfilename.at(EPSfilename.length()-1) == '\"'))
     EPSfilename = EPSfilename.mid(1,EPSfilename.length()-2);
-  }
+
+  // If the file name ends in 'png', 'gif', 'jpg' or 'jpeg', we assume
+  // that this is NOT a PostScript file, and we exit here. The graphic
+  // file is later read when the page is rendered.
+  QString ending = EPSfilename.section('.', -1).lower();
+  if ((ending == "png") || (ending == "gif") || (ending == "jpg") || (ending == "jpeg") || (ending == "mng"))
+    return;
 
   QString originalFName = EPSfilename;
 
@@ -204,10 +211,10 @@ void dviWindow::prescan_embedPS(char *cp, Q_UINT8 *beginningOfSpecialCommand)
 }
 
 
-void dviWindow::prescan_removePageSizeInfo(char *cp, Q_UINT8 *beginningOfSpecialCommand)
+void dviRenderer::prescan_removePageSizeInfo(char *cp, Q_UINT8 *beginningOfSpecialCommand)
 {
 #ifdef  DEBUG_PRESCAN
-  kdDebug(4300) << "dviWindow::prescan_embedPS( cp = " << cp << " ) " << endl;
+  kdDebug(4300) << "dviRenderer::prescan_embedPS( cp = " << cp << " ) " << endl;
 #endif
   
   // Encapsulated Postscript File
@@ -219,9 +226,9 @@ void dviWindow::prescan_removePageSizeInfo(char *cp, Q_UINT8 *beginningOfSpecial
 }
 
 
-void dviWindow::prescan_ParsePapersizeSpecial(QString cp)
+void dviRenderer::prescan_ParsePapersizeSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdDebug(4300) << "Papersize-Special : papersize" << cp << endl;
 #endif
 
@@ -231,9 +238,6 @@ void dviWindow::prescan_ParsePapersizeSpecial(QString cp)
     cp = cp.mid(1);
     dviFile->suggestedPageSize = new pageSize;
     dviFile->suggestedPageSize->setPageSize(cp);
-#ifdef DEBUG_SPECIAL
-    kdDebug(4300) << "Suggested paper size is " << dviFile->suggestedPageSize.serialize() << "." << endl;
-#endif
   } else 
     printErrorMsgForSpecials(i18n("The papersize data '%1' could not be parsed.").arg(cp));
 
@@ -241,7 +245,7 @@ void dviWindow::prescan_ParsePapersizeSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParseBackgroundSpecial(QString cp)
+void dviRenderer::prescan_ParseBackgroundSpecial(QString cp)
 {
   QColor col = parseColorSpecification(cp.stripWhiteSpace());
   if (col.isValid())
@@ -251,20 +255,16 @@ void dviWindow::prescan_ParseBackgroundSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParseHTMLAnchorSpecial(QString cp)
+void dviRenderer::prescan_ParseHTMLAnchorSpecial(QString cp)
 {
   cp.truncate(cp.find('"'));
-#ifdef DEBUG_SPECIAL
-  kdDebug(4300) << "HTML-special, anchor " << cp.latin1() << endl;
-  kdDebug(4300) << "page " << current_page << endl;
-#endif
-  anchorList[cp] = DVI_Anchor(current_page, currinf.data.dvi_v);
+  anchorList[cp] = anchor(current_page+1, currinf.data.dvi_v/(resolutionInDPI*shrinkfactor));
 }
 
 
-void dviWindow::prescan_ParsePSHeaderSpecial(QString cp)
+void dviRenderer::prescan_ParsePSHeaderSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdDebug(4300) << "PostScript-special, header " << cp.latin1() << endl;
 #endif
 
@@ -273,9 +273,9 @@ void dviWindow::prescan_ParsePSHeaderSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParsePSBangSpecial(QString cp)
+void dviRenderer::prescan_ParsePSBangSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdDebug(4300) << "PostScript-special, literal header " << cp.latin1() << endl;
 #endif
   
@@ -285,15 +285,15 @@ void dviWindow::prescan_ParsePSBangSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParsePSQuoteSpecial(QString cp)
+void dviRenderer::prescan_ParsePSQuoteSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdError(4300) << "PostScript-special, literal PostScript " << cp.latin1() << endl;
 
 #endif
   
-  double PS_H = (currinf.data.dvi_h*300.0)/(65536*MFResolutions[font_pool.getMetafontMode()])-300;
-  double PS_V = (currinf.data.dvi_v*300.0)/MFResolutions[font_pool.getMetafontMode()] - 300;
+  double PS_H = (currinf.data.dvi_h*300.0)/(65536*1200)-300;
+  double PS_V = (currinf.data.dvi_v*300.0)/1200 - 300;
   PostScriptOutPutString->append( QString(" %1 %2 moveto\n").arg(PS_H).arg(PS_V) );
   PostScriptOutPutString->append( " @beginspecial @setspecial \n" );
   PostScriptOutPutString->append( cp );
@@ -301,9 +301,9 @@ void dviWindow::prescan_ParsePSQuoteSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParsePSSpecial(QString cp)
+void dviRenderer::prescan_ParsePSSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdDebug(4300) << "PostScript-special, direct PostScript " << cp << endl;
 #endif
 
@@ -330,15 +330,15 @@ void dviWindow::prescan_ParsePSSpecial(QString cp)
     if (cp.startsWith("ps:SDict begin [") && cp.endsWith(" pdfmark end")) {  // hyperref definition of link/anchor/bookmark/etc
       if (cp.contains("/DEST")) { // The PostScript code defines an anchor
 	QString anchorName = cp.section('(', 1, 1).section(')', 0, 0);
-	anchorList[anchorName] = DVI_Anchor(current_page, currinf.data.dvi_v);
+	anchorList[anchorName] = anchor(current_page+1, currinf.data.dvi_v/(resolutionInDPI*shrinkfactor));
       }
       return;
     }
   }
   
   
-  double PS_H = (currinf.data.dvi_h*300.0)/(65536*MFResolutions[font_pool.getMetafontMode()])-300;
-  double PS_V = (currinf.data.dvi_v*300.0)/MFResolutions[font_pool.getMetafontMode()] - 300;
+  double PS_H = (currinf.data.dvi_h*300.0)/(65536*1200)-300;
+  double PS_V = (currinf.data.dvi_v*300.0)/1200 - 300;
   
   if (cp.find("ps::[begin]", 0, false) == 0) {
     PostScriptOutPutString->append( QString(" %1 %2 moveto\n").arg(PS_H).arg(PS_V) );
@@ -358,16 +358,14 @@ void dviWindow::prescan_ParsePSSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParsePSFileSpecial(QString cp)
+void dviRenderer::prescan_ParsePSFileSpecial(QString cp)
 {
-#ifdef DEBUG_SPECIAL
+#ifdef DEBUG_PRESCAN
   kdDebug(4300) << "epsf-special: psfile=" << cp <<endl;
 #endif
 
   QString include_command = cp.simplifyWhiteSpace();
 
-  dviFile->numberOfExternalPSFiles++;  
-
   // The line is supposed to start with "..ile=", and then comes the
   // filename. Figure out what the filename is and stow it away. Of
   // course, this does not work if the filename contains spaces
@@ -384,6 +382,17 @@ void dviWindow::prescan_ParsePSFileSpecial(QString cp)
     EPSfilename = EPSfilename.mid(1,EPSfilename.length()-2);
   }
 
+  // If the file name ends in 'png', 'gif', 'jpg' or 'jpeg', we assume
+  // that this is NOT a PostScript file, and we exit here.
+  QString ending = EPSfilename.section('.', -1).lower();
+  if ((ending == "png") || (ending == "gif") || (ending == "jpg") || (ending == "jpeg")) {
+    dviFile->numberOfExternalNONPSFiles++;  
+    return;
+  }
+  
+  // Now assume that the graphics file *is* a PostScript file
+  dviFile->numberOfExternalPSFiles++;  
+  
   // Now locate the Gfx file on the hard disk...
   EPSfilename = ghostscript_interface::locateEPSfile(EPSfilename, dviFile);
   
@@ -407,9 +416,10 @@ void dviWindow::prescan_ParsePSFileSpecial(QString cp)
   parse_special_argument(include_command, "rhi=", &rhi);
   parse_special_argument(include_command, "angle=", &angle);
 
+
   if (QFile::exists(EPSfilename)) {
-    double PS_H = (currinf.data.dvi_h*300.0)/(65536*MFResolutions[font_pool.getMetafontMode()])-300;
-    double PS_V = (currinf.data.dvi_v*300.0)/MFResolutions[font_pool.getMetafontMode()] - 300;
+    double PS_H = (currinf.data.dvi_h*300.0)/(65536*1200)-300;
+    double PS_V = (currinf.data.dvi_v*300.0)/1200 - 300;
     PostScriptOutPutString->append( QString(" %1 %2 moveto\n").arg(PS_H).arg(PS_V) );
     PostScriptOutPutString->append( "@beginspecial " );
     PostScriptOutPutString->append( QString(" %1 @llx").arg(llx) );
@@ -431,7 +441,7 @@ void dviWindow::prescan_ParsePSFileSpecial(QString cp)
 }
 
 
-void dviWindow::prescan_ParseSourceSpecial(QString cp)
+void dviRenderer::prescan_ParseSourceSpecial(QString cp)
 {
   // if no rendering takes place, i.e. when the DVI file is first
   // loaded, generate a DVI_SourceFileAnchor. These anchors are used
@@ -446,12 +456,12 @@ void dviWindow::prescan_ParseSourceSpecial(QString cp)
   Q_UINT32 sourceLineNumber = cp.left(j).toUInt();
   QFileInfo fi1(dviFile->filename);
   QString  sourceFileName   = QFileInfo(fi1.dir(), cp.mid(j).stripWhiteSpace()).absFilePath();
-  DVI_SourceFileAnchor sfa(sourceFileName, sourceLineNumber, current_page, currinf.data.dvi_v);
+  DVI_SourceFileAnchor sfa(sourceFileName, sourceLineNumber, current_page+1, currinf.data.dvi_v/(resolutionInDPI*shrinkfactor));
   sourceHyperLinkAnchors.push_back(sfa);
 }
 
 
-void dviWindow::prescan_parseSpecials(char *cp, Q_UINT8 *)
+void dviRenderer::prescan_parseSpecials(char *cp, Q_UINT8 *)
 {
   QString special_command(cp);
 
@@ -525,47 +535,49 @@ void dviWindow::prescan_parseSpecials(char *cp, Q_UINT8 *)
 }
 
 
-void dviWindow::prescan_setChar(unsigned int ch)
+void dviRenderer::prescan_setChar(unsigned int ch)
 {
   TeXFontDefinition *fontp = currinf.fontp;
   if (fontp == NULL)
     return;
 
-  if (currinf.set_char_p == &dviWindow::set_char) {
+  if (currinf.set_char_p == &dviRenderer::set_char) {
     glyph *g = ((TeXFont *)(currinf.fontp->font))->getGlyph(ch, true, globalColor);
     if (g == NULL)
       return;
     currinf.data.dvi_h += (int)(currinf.fontp->scaled_size_in_DVI_units * dviFile->getCmPerDVIunit() * 
-				(MFResolutions[font_pool.getMetafontMode()] / 2.54)/16.0 * g->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
+				(1200.0 / 2.54)/16.0 * g->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
     return;
   }
  
-  if (currinf.set_char_p == &dviWindow::set_vf_char) {
+  if (currinf.set_char_p == &dviRenderer::set_vf_char) {
     macro *m = &currinf.fontp->macrotable[ch];
     if (m->pos == NULL) 
       return;
     currinf.data.dvi_h += (int)(currinf.fontp->scaled_size_in_DVI_units * dviFile->getCmPerDVIunit() * 
-				(MFResolutions[font_pool.getMetafontMode()] / 2.54)/16.0 * m->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
+				(1200.0 / 2.54)/16.0 * m->dvi_advance_in_units_of_design_size_by_2e20 + 0.5);
     return;
   }
 }
 
 
-void dviWindow::prescan(parseSpecials specialParser)
+void dviRenderer::prescan(parseSpecials specialParser)
 {
 #ifdef DEBUG_PRESCAN
-  kdDebug(4300) << "dviWindow::prescan( ... )" << endl;
+  kdDebug(4300) << "dviRenderer::prescan( ... )" << endl;
 #endif
-  
+
+  if (resolutionInDPI == 0.0)
+    setResolution(100);
+
   Q_INT32 RRtmp=0, WWtmp=0, XXtmp=0, YYtmp=0, ZZtmp=0;
   Q_UINT8 ch;
-  double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 
-    MFResolutions[font_pool.getMetafontMode()]/2.54;
+  double fontPixelPerDVIunit = dviFile->getCmPerDVIunit() * 1200.0/2.54;
 
   stack.clear();
 
   currinf.fontp        = NULL;
-  currinf.set_char_p   = &dviWindow::set_no_char;
+  currinf.set_char_p   = &dviRenderer::set_no_char;
 
   for (;;) {
     ch = readUINT8();
@@ -613,8 +625,8 @@ void dviWindow::prescan(parseSpecials specialParser)
       
     case BOP:
       command_pointer += 11 * 4;
-      currinf.data.dvi_h = MFResolutions[font_pool.getMetafontMode()] << 16; // Reminder: DVI-coordinates start at (1",1") from top of page
-      currinf.data.dvi_v = MFResolutions[font_pool.getMetafontMode()];
+      currinf.data.dvi_h = 1200 << 16; // Reminder: DVI-coordinates start at (1",1") from top of page
+      currinf.data.dvi_v = 1200;
       currinf.data.pxl_v = int(currinf.data.dvi_v/shrinkfactor);
       currinf.data.w = currinf.data.x = currinf.data.y = currinf.data.z = 0;
       break;
